==> spades_3.13.1%2Bdfsg.orig.tar.xz: spades-3.13.1/assembler/src/common/utils/ph_map/storing_traits.hpp <==
//***************************************************************************
//* Copyright (c) 2015 Saint Petersburg State University
//* Copyright (c) 2011-2014 Saint Petersburg Academic University
//* All Rights Reserved
//* See file LICENSE for details.
//***************************************************************************

#pragma once

#include "values.hpp"
#include "utils/verify.hpp"

namespace utils {

struct SimpleStoring {
    template<class K, class V>
    static V get_value(const ValueArray<V> &values, const K& key) {
        return values[key.idx()];
    }

    template<class K, class V>
    static void set_value(ValueArray<V> &values, const K& key, const V& value) {
        values[key.idx()] = value;
    }

    static bool IsInvertable() {
        return false;
    }
};

struct InvertableStoring {
    template<typename V>
    struct default_inverter {
        template<typename K>
        V operator()(const V& v, const K& k) const {
            return v.conjugate(k);
        }
    };

    template<typename V>
    struct trivial_inverter {
        template<typename K>
        V operator()(const V& v, const K& /*k*/) const {
            return v;
        }
    };

    template<class K, class V, class F = default_inverter<V>>
    static V get_value(const ValueArray<V> &values, const K& key,
                       const F& inverter = F()) {
        if (key.is_minimal())
            return values[key.idx()];
        else
            return inverter(values[key.idx()], key);
    }

    template<class K, class V, class F = default_inverter<V>>
    static void set_value(ValueArray<V>& values, const K& key, const V& value,
                          const F& inverter = F()) {
        VERIFY(key.idx() < values.size());
        if (key.is_minimal()) {
            values[key.idx()] = value;
        } else {
            values[key.idx()] = inverter(value, key);
        }
    }

    static bool IsInvertable() {
        return true;
    }
};

typedef InvertableStoring DefaultStoring;

//FIXME make functor
template<class StoringType>
struct StoringTypeFilter {
};

template<>
struct StoringTypeFilter<SimpleStoring> {
    template<class Kmer>
    bool filter(const Kmer &/*kmer*/) const {
        return true;
    }
};

template<>
struct StoringTypeFilter<InvertableStoring> {
    template<class Kmer>
    bool filter(const Kmer &kmer) const {
        return kmer.IsMinimal();
    }
};

}

==> spades_3.15.4%2Bdfsg.orig.tar.xz: spades-3.15.4/assembler/src/common/utils/ph_map/storing_traits.hpp <==
//***************************************************************************
//* Copyright (c) 2015 Saint Petersburg State University
//* Copyright (c) 2011-2014 Saint Petersburg Academic University
//* All Rights Reserved
//* See file LICENSE for details.
//***************************************************************************

#pragma once

#include "utils/verify.hpp"

namespace utils {

struct SimpleStoring {
    template<class K, class Container>
    static auto get_value(const Container &values, const K& key) {
        return values[key.idx()];
    }

    template<class K, class V, class Container>
    static void set_value(Container &values, const K& key, const V& value) {
        values[key.idx()] = value;
    }

    static constexpr bool IsInvertable() {
        return false;
    }
};

struct InvertableStoring {
    struct default_inverter {
        template<typename K, typename V>
        V operator()(const V& v, const K& k) const {
            return v.conjugate(k);
        }
    };

    struct trivial_inverter {
        template<typename K, typename V>
        V operator()(const V& v, const K& /*k*/) const {
            return v;
        }
    };

    template<class K, class Container, class F = default_inverter>
    static auto get_value(const Container &values, const K& key,
                          const F& inverter = F()) {
        if (key.is_minimal())
            return values[key.idx()];
        else
            return inverter(values[key.idx()], key);
    }

    template<class K, class V, class Container, class F = default_inverter>
    static void set_value(Container& values, const K& key, const V& value,
                          const F& inverter = F()) {
        VERIFY(key.idx() < values.size());
        if (key.is_minimal()) {
            values[key.idx()] = value;
        } else {
            values[key.idx()] = inverter(value, key);
        }
    }

    static constexpr bool IsInvertable() {
        return true;
    }
};

typedef InvertableStoring DefaultStoring;

//FIXME make functor
template<class StoringType>
struct StoringTypeFilter {
};

template<>
struct StoringTypeFilter<SimpleStoring> {
    template<class Kmer>
    bool operator()(const Kmer &/*kmer*/) const {
        return true;
    }

    template<class Kmer>
    static bool filter(const Kmer &/*kmer*/) {
        return true;
    }
};

template<>
struct StoringTypeFilter<InvertableStoring> {
    template<class Kmer>
    bool operator()(const Kmer &kmer) const {
        return filter(kmer);
    }

    template<class Kmer>
    static bool filter(const Kmer &kmer) {
        return kmer.IsMinimal();
    }
};

}
